"use strict";
// Copyright (c) 2021-2023 Peter Matta
//
// This source code is licensed under the MIT license found in the
// LICENSE file in the root directory of this source tree.
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fp4tsContentTypeHeader = exports.fp4tsChallenge = exports.fp4tsBasicCredentials = exports.fp4tsAuthorizationHeader = exports.fp4tsCredentials = exports.fp4tsAccessControlAllowMethodHeader = exports.fp4tsAcceptHeader = exports.fp4tsMediaRange = exports.fp4tsMediaType = exports.fp4tsQValue = exports.fp4tsMethod = exports.fp4tsStatus = exports.fp4tsValidStatusCode = void 0;
const fast_check_1 = __importDefault(require("fast-check"));
const core_1 = require("@fp4ts/core");
const http_core_1 = require("@fp4ts/http-core");
const collections_1 = require("@fp4ts/collections");
const A = __importStar(require("@fp4ts/collections-test-kit/lib/arbitraries"));
__exportStar(require("@fp4ts/stream-test-kit/lib/arbitraries"), exports);
const fp4tsValidStatusCode = () => fast_check_1.default.integer({ min: http_core_1.Status.MinCode, max: http_core_1.Status.MaxCode });
exports.fp4tsValidStatusCode = fp4tsValidStatusCode;
const fp4tsStatus = () => (0, exports.fp4tsValidStatusCode)().map(code => http_core_1.Status.unsafeFromCode(code));
exports.fp4tsStatus = fp4tsStatus;
const fp4tsMethod = () => fast_check_1.default.constantFrom(...http_core_1.Method.all);
exports.fp4tsMethod = fp4tsMethod;
const arbChar = fast_check_1.default
    .integer({ min: 0x00, max: 0x07f })
    .map(x => core_1.Char.fromByte(x));
const ctlChar = [
    core_1.Char.fromByte(0x07f),
    ...[...new Array(0x1f).keys()].map(x => core_1.Char.fromByte(x)),
];
const arbCtlChar = fast_check_1.default.constantFrom(...ctlChar);
const lws = [' ', '\t'];
const arbCrLf = fast_check_1.default.constant('\r\n');
const arbRightLws = fast_check_1.default
    .array(fast_check_1.default.constantFrom(...lws), { minLength: 1 })
    .map(xs => xs.join(''));
const arbLws = fast_check_1.default.oneof(fast_check_1.default.array(fast_check_1.default.oneof(arbCrLf, arbRightLws)).map(x => x.join('')), arbRightLws);
const octets = [...new Array(0xff).keys()].map(x => core_1.Char.fromByte(x));
const arbOctet = fast_check_1.default.constantFrom(...octets);
const allowedText = octets.filter(x => !ctlChar.includes(x));
const arbText = fast_check_1.default.oneof(fast_check_1.default
    .array(fast_check_1.default.constantFrom(...allowedText), { minLength: 1 })
    .map(x => x.join('')), arbLws);
const allowedQDText = allowedText.filter(c => c !== '"' && c !== '\\');
const arbQDText = fast_check_1.default
    .array(fast_check_1.default.constantFrom(...allowedQDText), { minLength: 1 })
    .map(xs => xs.join(''));
// prettier-ignore
const tchars = [
    '!', '#', '$', '%', '&', "'", '*', '+', '-', '.', '^', '_', '`', '|', '~',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
];
const arbTchar = fast_check_1.default.constantFrom(...tchars);
const arbToken = fast_check_1.default.array(arbTchar, { minLength: 1 }).map(xs => xs.join(''));
const vToken = [...new Array(0x07e - 0x021 + 1).keys()].map(x => core_1.Char.fromByte((x + 0x21)));
const arbVToken = fast_check_1.default.constantFrom(...vToken);
const arbFieldVchar = arbVToken;
const fp4tsQValue = () => fast_check_1.default
    .oneof(fast_check_1.default.constantFrom(0, 1000), fast_check_1.default.integer({ min: 0, max: 1000 }))
    .map(xs => http_core_1.QValue.fromThousands(xs).get);
exports.fp4tsQValue = fp4tsQValue;
const allowedQuotedText = allowedText.map(c => c === '"' ? '\\"' : c === '\\' ? c === '\\\\' : c);
const arbQuotedText = fast_check_1.default
    .array(fast_check_1.default.constantFrom(...allowedQuotedText), { minLength: 1 })
    .map(s => `"${s.join('')}"`);
const arbUnquotedText = arbQDText;
const arbMediaRangeExtension = fast_check_1.default.tuple(arbToken, fast_check_1.default.oneof(arbQDText, arbText));
const arbMediaRangeExtensions = fast_check_1.default
    .array(arbMediaRangeExtension)
    .map(xs => collections_1.OrdMap.fromArray(xs));
const fp4tsMediaType = () => fast_check_1.default.constantFrom(...http_core_1.MediaType.all.toArray.map(core_1.snd));
exports.fp4tsMediaType = fp4tsMediaType;
const fp4tsMediaRange = () => fast_check_1.default
    .tuple(arbToken.map(xs => xs.toLowerCase()), arbMediaRangeExtensions)
    .map(([type, extensions]) => new http_core_1.MediaRange(type, extensions));
exports.fp4tsMediaRange = fp4tsMediaRange;
const arbMediaRangeAndQValue = fast_check_1.default
    .tuple((0, exports.fp4tsMediaRange)(), (0, exports.fp4tsQValue)())
    .map(([mr, qv]) => new http_core_1.MediaRangeAndQValue(mr, qv));
const fp4tsAcceptHeader = () => A.fp4tsNel(arbMediaRangeAndQValue).map(xs => new http_core_1.Accept(xs));
exports.fp4tsAcceptHeader = fp4tsAcceptHeader;
const fp4tsAccessControlAllowMethodHeader = () => (0, exports.fp4tsMethod)().map(m => new http_core_1.AccessControlAllowMethod(m));
exports.fp4tsAccessControlAllowMethodHeader = fp4tsAccessControlAllowMethodHeader;
const arbAuthScheme = fast_check_1.default.constantFrom(http_core_1.AuthScheme.Basic, http_core_1.AuthScheme.Bearer, http_core_1.AuthScheme.Digest, http_core_1.AuthScheme.OAuth);
const arbT68Char = fast_check_1.default
    .char()
    .filter(c => /\W-\._+\//.test(c));
const arbAuthToken = fast_check_1.default
    .tuple(arbAuthScheme, arbT68Char)
    .map(([scheme, tok]) => new http_core_1.Token(scheme, tok));
const arbAuthParam = fast_check_1.default.tuple(arbToken, fast_check_1.default.oneof(arbQDText, arbQuotedText));
const arbAuthParams_ = fast_check_1.default
    .array(arbAuthParam, { minLength: 1 })
    .map(xs => collections_1.List.fromArray(xs));
const arbAuthParams = fast_check_1.default
    .tuple(arbAuthScheme, arbAuthParams_)
    .map(([scheme, params]) => new http_core_1.AuthParams(scheme, params));
const fp4tsCredentials = () => fast_check_1.default.oneof(arbAuthToken, arbAuthParams);
exports.fp4tsCredentials = fp4tsCredentials;
const fp4tsAuthorizationHeader = () => (0, exports.fp4tsCredentials)().map(creds => new http_core_1.Authorization(creds));
exports.fp4tsAuthorizationHeader = fp4tsAuthorizationHeader;
const fp4tsBasicCredentials = () => fast_check_1.default
    .tuple(fast_check_1.default.unicodeString(), fast_check_1.default.unicodeString())
    .map(([username, password]) => new http_core_1.BasicCredentials(username, password));
exports.fp4tsBasicCredentials = fp4tsBasicCredentials;
// export const fp4tsCredentials = (): Arbitrary<Credentials> =>
//   fc.oneof(arbAuthParams, arbBasicCredentials);
const arbRealm = arbToken;
const arbChallengeParam = fast_check_1.default
    .tuple(arbToken, fast_check_1.default.oneof(arbQDText, arbText))
    .filter(x => x[0] !== 'realm');
const arbChallengeParams = fast_check_1.default
    .array(arbChallengeParam)
    .map(xs => collections_1.OrdMap.fromArray(xs));
const fp4tsChallenge = () => fast_check_1.default
    .tuple(arbAuthScheme, arbRealm, arbChallengeParams)
    .map(([scheme, realm, params]) => new http_core_1.Challenge(scheme, realm, params));
exports.fp4tsChallenge = fp4tsChallenge;
const fp4tsContentTypeHeader = () => (0, exports.fp4tsMediaType)().map(mt => new http_core_1.ContentType(mt));
exports.fp4tsContentTypeHeader = fp4tsContentTypeHeader;
//# sourceMappingURL=arbitraries.js.map